!
!  procedures.f90
!
!  Free-Format Fortran Source File 
!  Generated by PGI Visual Fortran(R)
!  4/21/2011 11:03:58 AM
!
module procedures
USE nrtype

implicit none

contains

!-------------------------------------------------------------------------------!

subroutine cumnor ( arg, cum )

!*****************************************************************************80
!
!! CUMNOR computes the cumulative normal distribution.
!
!  Discussion:
!
!    This function evaluates the normal distribution function:
!
!                              / x
!                     1       |       -t*t/2
!          P(x) = ----------- |      e       dt
!                 sqrt(2 pi)  |
!                             /-oo
!
!    This transportable program uses rational functions that
!    theoretically approximate the normal distribution function to
!    at least 18 significant decimal digits.  The accuracy achieved
!    depends on the arithmetic system, the compiler, the intrinsic
!    functions, and proper selection of the machine dependent
!    constants.
!
!  Parameters:
!
!    Input, real ( dp ) ARG, the upper limit of integration.
!
!    Output, real ( dp ) CUM, the Normal density CDF
!
!  Local Parameters:
!
!    Local, real ( dp ) EPS, the argument below which anorm(x) 
!    may be represented by 0.5 and above which  x*x  will not underflow.
!    A conservative value is the largest machine number X
!    such that   1.0D+00 + X = 1.0D+00   to machine precision.
!
  implicit none

  real ( dp ), parameter, dimension ( 5 ) :: a = (/ &
    2.2352520354606839287D+00, &
    1.6102823106855587881D+02, &
    1.0676894854603709582D+03, &
    1.8154981253343561249D+04, &
    6.5682337918207449113D-02 /)
  real ( dp ) arg
  real ( dp ), parameter, dimension ( 4 ) :: b = (/ &
    4.7202581904688241870D+01, &
    9.7609855173777669322D+02, &
    1.0260932208618978205D+04, &
    4.5507789335026729956D+04 /)
  real ( dp ), parameter, dimension ( 9 ) :: c = (/ &
    3.9894151208813466764D-01, &
    8.8831497943883759412D+00, &
    9.3506656132177855979D+01, &
    5.9727027639480026226D+02, &
    2.4945375852903726711D+03, &
    6.8481904505362823326D+03, &
    1.1602651437647350124D+04, &
    9.8427148383839780218D+03, &
    1.0765576773720192317D-08 /)
  real ( dp ) cum
  real ( dp ), parameter, dimension ( 8 ) :: d = (/ &
    2.2266688044328115691D+01, &
    2.3538790178262499861D+02, &
    1.5193775994075548050D+03, &
    6.4855582982667607550D+03, &
    1.8615571640885098091D+04, &
    3.4900952721145977266D+04, &
    3.8912003286093271411D+04, &
    1.9685429676859990727D+04 /)
  real ( dp ) del
  real ( dp ) eps
  integer i
  real ( dp ), parameter, dimension ( 6 ) :: p = (/ &
    2.1589853405795699D-01, &
    1.274011611602473639D-01, &
    2.2235277870649807D-02, &
    1.421619193227893466D-03, &
    2.9112874951168792D-05, &
    2.307344176494017303D-02 /)
  real ( dp ), parameter, dimension ( 5 ) :: q = (/ &
    1.28426009614491121D+00, &
    4.68238212480865118D-01, &
    6.59881378689285515D-02, &
    3.78239633202758244D-03, &
    7.29751555083966205D-05 /)
  real ( dp ), parameter :: root32 = 5.656854248D+00
  real ( dp ), parameter :: sixten = 16.0D+00
  real ( dp ) temp
  real ( dp ), parameter :: sqrpi = 3.9894228040143267794D-01
  real ( dp ), parameter :: thrsh = 0.66291D+00
  real ( dp ) x
  real ( dp ) xden
  real ( dp ) xnum
  real ( dp ) y
  real ( dp ) xsq
!
!  Machine dependent constants
!
  eps = epsilon ( 1.0D+00 ) * 0.5D+00

  x = arg
  y = abs ( x )

  if ( y <= thrsh ) then
!
!  Evaluate  anorm  for  |X| <= 0.66291
!
    if ( eps < y ) then
      xsq = x * x
    else
      xsq = 0.0D+00
    end if

    xnum = a(5) * xsq
    xden = xsq
    do i = 1, 3
      xnum = ( xnum + a(i) ) * xsq
      xden = ( xden + b(i) ) * xsq
    end do
    cum = x * ( xnum + a(4) ) / ( xden + b(4) )
    temp = cum
    cum = 0.5D+00 + temp
!
!  Evaluate ANORM for 0.66291 <= |X| <= sqrt(32)
!
  else if ( y <= root32 ) then

    xnum = c(9) * y
    xden = y
    do i = 1, 7
      xnum = ( xnum + c(i) ) * y
      xden = ( xden + d(i) ) * y
    end do
    cum = ( xnum + c(8) ) / ( xden + d(8) )
    xsq = aint ( y * sixten ) / sixten
    del = ( y - xsq ) * ( y + xsq )
    cum = exp ( - xsq * xsq * 0.5D+00 ) * exp ( -del * 0.5D+00 ) * cum

    if ( 0.0D+00 < x ) then
      cum = 1.0D+00 - cum
    end if
!
!  Evaluate ANORM for sqrt(32) < |X|.
!
  else

    cum = 0.0D+00
    xsq = 1.0D+00 / ( x * x )
    xnum = p(6) * xsq
    xden = xsq
    do i = 1, 4
      xnum = ( xnum + p(i) ) * xsq
      xden = ( xden + q(i) ) * xsq
    end do

    cum = xsq * ( xnum + p(5) ) / ( xden + q(5) )
    cum = ( sqrpi - cum ) / y
    xsq = aint ( x * sixten ) / sixten
    del = ( x - xsq ) * ( x + xsq )
    cum = exp ( - xsq * xsq * 0.5D+00 ) &
      * exp ( - del * 0.5D+00 ) * cum

    if ( 0.0D+00 < x ) then
      cum = 1.0D+00 - cum
    end if

  end if

  if ( cum < tiny ( cum ) ) then
    cum = 0.0D+00
  end if

  return
end subroutine

function normcdfinv ( p )

!*****************************************************************************80
!
!! NORMCDFINV inverts the standard normal CDF.
!
!  Discussion:
!
!    The result is accurate to about 1 part in 10**16.
!
!  Parameters:
!
!    Input, real ( dp ) P, the value of the cumulative probability 
!    densitity function.  0 < P < 1.  If P is outside this range,
!    an "infinite" value will be returned.
!
!    Output, real ( dp ) D_NORMAL_01_CDF_INVERSE, the normal deviate 
!    value with the property that the probability of a standard normal
!    deviate being less than or equal to the value is P.
!
  implicit none

  real    ( dp ), parameter, dimension ( 8 ) :: a = (/ &
    3.3871328727963666080D+00, &
    1.3314166789178437745D+02, &
    1.9715909503065514427D+03, &
    1.3731693765509461125D+04, &
    4.5921953931549871457D+04, &
    6.7265770927008700853D+04, &
    3.3430575583588128105D+04, &
    2.5090809287301226727D+03 /)
  real    ( dp ), parameter, dimension ( 8 ) :: b = (/ &
    1.0D+00, &
    4.2313330701600911252D+01, &
    6.8718700749205790830D+02, &
    5.3941960214247511077D+03, &
    2.1213794301586595867D+04, &
    3.9307895800092710610D+04, &
    2.8729085735721942674D+04, &
    5.2264952788528545610D+03 /)
  real   ( dp ), parameter, dimension ( 8 ) :: c = (/ &
    1.42343711074968357734D+00, &
    4.63033784615654529590D+00, &
    5.76949722146069140550D+00, &
    3.64784832476320460504D+00, &
    1.27045825245236838258D+00, &
    2.41780725177450611770D-01, &
    2.27238449892691845833D-02, &
    7.74545014278341407640D-04 /)
  real    ( dp ), parameter :: const1 = 0.180625D+00
  real    ( dp ), parameter :: const2 = 1.6D+00
  real    ( dp ), parameter, dimension ( 8 ) :: d = (/ &
    1.0D+00, &
    2.05319162663775882187D+00, &
    1.67638483018380384940D+00, &
    6.89767334985100004550D-01, &
    1.48103976427480074590D-01, &
    1.51986665636164571966D-02, &
    5.47593808499534494600D-04, &
    1.05075007164441684324D-09 /)
  real    ( dp ), parameter, dimension ( 8 ) :: e = (/ &
    6.65790464350110377720D+00, &
    5.46378491116411436990D+00, &
    1.78482653991729133580D+00, &
    2.96560571828504891230D-01, &
    2.65321895265761230930D-02, &
    1.24266094738807843860D-03, &
    2.71155556874348757815D-05, &
    2.01033439929228813265D-07 /)
  real    ( dp ), parameter, dimension ( 8 ) :: f = (/ &
    1.0D+00, &
    5.99832206555887937690D-01, &
    1.36929880922735805310D-01, &
    1.48753612908506148525D-02, &
    7.86869131145613259100D-04, &
    1.84631831751005468180D-05, &
    1.42151175831644588870D-07, &
    2.04426310338993978564D-15 /)
  real    ( dp ) p
  real    ( dp ) q
  real    ( dp ) r
  real    ( dp ) normcdfinv
  real    ( dp ), parameter :: split1 = 0.425D+00
  real    ( dp ), parameter :: split2 = 5.0D+00

  if ( p <= 0.0D+00 ) then
    normcdfinv = - huge ( p )
    return
  end if

  if ( 1.0D+00 <= p ) then
    normcdfinv = huge ( p )
    return
  end if

  q = p - 0.5D+00

  if ( abs ( q ) <= split1 ) then

    r = const1 - q * q
    normcdfinv = q * poly_value ( 8, a, r ) &
                                 / poly_value ( 8, b, r )

  else

    if ( q < 0.0D+00 ) then
      r = p
    else
      r = 1.0D+00 - p
    end if

    if ( r <= 0.0D+00 ) then
      normcdfinv = - 1.0D+00
      stop
    end if

    r = sqrt ( -log ( r ) )

    if ( r <= split2 ) then

      r = r - const2
      normcdfinv = poly_value ( 8, c, r ) &
                               / poly_value ( 8, d, r )

    else

      r = r - split2
      normcdfinv = poly_value ( 8, e, r ) &
                               / poly_value ( 8, f, r )
   
    end if

    if ( q < 0.0D+00 ) then
      normcdfinv = - normcdfinv
    end if

  end if

  return
end  function


function poly_value ( n, a, x )

!*****************************************************************************80
!
!! POLY_VALUE evaluates an R8POLY
!
!  Discussion:
!
!    For sanity's sake, the value of N indicates the NUMBER of 
!    coefficients, or more precisely, the ORDER of the polynomial,
!    rather than the DEGREE of the polynomial.  The two quantities
!    differ by 1, but cause a great deal of confusion.
!
!    Given N and A, the form of the polynomial is:
!
!      p(x) = a(1) + a(2) * x + ... + a(n-1) * x^(n-2) + a(n) * x^(n-1)
!
!  Parameters:
!
!    Input, integer ( kind = 4 ) N, the order of the polynomial.
!
!    Input, real ( dp ) A(N), the coefficients of the polynomial.
!    A(1) is the constant term.
!
!    Input, real ( dp ) X, the point at which the polynomial is 
!    to be evaluated.
!
!    Output, real ( dp ) R8POLY_VALUE, the value of the polynomial at X.
!
  implicit none

  integer ( kind = 4 ) n

  real    ( dp ) a(n)
  integer ( kind = 4 ) i
  real    ( dp ) poly_value
  real    ( dp ) x

  poly_value = 0.0D+00
  do i = n, 1, -1
    poly_value = poly_value * x + a(i)
  end do

  return
end function


subroutine gaus8 ( func, a, b, result,ybar,jpr,mu,sigmam,sigmae,rhom,N)

!*****************************************************************************80
!
!! GAUS8 estimates the integral of a function.
!
!  Discussion:
!
!    GAUS8 integrates real functions of one variable over finite
!    intervals using an adaptive 8-point Legendre-Gauss
!    algorithm.
!
!    GAUS8 is intended primarily for high accuracy integration or
!    integration of smooth functions.
!
!  Parameters:
!
!    Input, real ( dp ), external FUNC, name of external function to
!    be integrated.  This name must be in an external statement in the
!    calling program.  FUNC must be a function of one real argument.  The value
!    of the argument to FUNC is the variable of integration
!    which ranges from A to B.
!
!    Input, real ( dp ) A, the lower limit of integration.
!
!    Input, real ( dp ) B, the upper limit of integration.
!
!    Input/output, real ( dp ) ERR.
!    On input, ERR is a requested pseudorelative error
!    tolerance.  Normally pick a value of ABS ( ERR ) so that
!    STOL < ABS ( ERR ) <= 1.0D-3 where STOL is the single precision
!    unit roundoff.
!    RESULT will normally have no more error than
!    ABS ( ERR ) times the integral of the absolute value of
!    FUN(X).  Usually, smaller values for ERR yield more
!    accuracy and require more function evaluations.
!    A negative value for ERR causes an estimate of the
!    absolute error in RESULT to be returned in ERR.  Note that
!    ERR must be a variable (not a constant) in this case.
!    Note also that the user must reset the value of ERR
!    before making any more calls that use the variable ERR.
!    On output, ERR will be an estimate of the absolute error
!    in RESULT if the input value of ERR was negative.  ERR is
!    unchanged if the input value of ERR was non-negative.
!    The estimated error is solely for information to the user
!    and should not be used as a correction to the computed integral.
!
!    Output, real ( dp ) RESULT, the computed value of the integral.
!
!    Output, integer IERR, a status code.
!    Normal Codes:
!     1 RESULT most likely meets requested error tolerance, or A = B.
!    -1 A and B are too nearly equal to allow normal
!        integration.  RESULT is set to zero.
!     Abnormal Code:
!     2 RESULT probably does not meet requested error tolerance.
!
  implicit none
  INTEGER, INTENT(IN) :: jpr,N
  REAL ( dp ), INTENT(IN) :: mu,sigmam,sigmae,rhom,ybar(N+1)
  real ( dp ), INTENT(IN):: a
  real ( dp ) aa(30)
  real ( dp ) ae
  real ( dp ) anib
  real ( dp ) area
  real ( dp ), INTENT(IN):: b
  real ( dp ) c
  real ( dp ) ce
  real ( dp ) ee
  real ( dp ) ef
  real ( dp ) eps
  real ( dp ) errt
  real ( dp ) est
  real ( dp ), external :: func
  real ( dp ) g8
  real ( dp ) gl
  real ( dp ) glr
  real ( dp ) gr(30)
  real ( dp ) h
  real ( dp ) hh(30)
  integer, save :: icall = 0
  integer ierr
  integer k
  integer, save :: kml = 6
  integer, save :: kmx = 5000
  integer l
  integer lmn
  integer lmx
  integer lr(30)
  integer mxl
  integer nbits
  integer nib
  integer, save :: nlmn = 1
  integer nlmx
  real ( dp ), INTENT(OUT):: result
  real ( dp ) tol
  real ( dp ) vl(30)
  real ( dp ) vr
  real ( dp ), save :: w1 = 3.62683783378361983D-01
  real ( dp ), save :: w2 = 3.13706645877887287D-01
  real ( dp ), save :: w3 = 2.22381034453374471D-01
  real ( dp ), save :: w4 = 1.01228536290376259D-01
  real ( dp ) x
  real ( dp ), save :: x1 = 1.83434642495649805D-01
  real ( dp ), save :: x2 = 5.25532409916328986D-01
  real ( dp ), save :: x3 = 7.96666477413626740D-01
  real ( dp ), save :: x4 = 9.60289856497536232D-01
!
!  Warning!  Statement function!
!
  g8(x,h) = h * ( ( & 
                w1 * ( func ( x - x1 * h,ybar,jpr,mu,sigmam,sigmae,rhom,N ) + func ( x + x1 * h,ybar,jpr,mu,sigmam,sigmae,rhom,N ) )   &
              + w2 * ( func ( x - x2 * h,ybar,jpr,mu,sigmam,sigmae,rhom,N ) + func ( x + x2 * h,ybar,jpr,mu,sigmam,sigmae,rhom,N ) ) ) &
            + ( w3 * ( func ( x - x3 * h,ybar,jpr,mu,sigmam,sigmae,rhom,N ) + func ( x + x3 * h,ybar,jpr,mu,sigmam,sigmae,rhom,N ) )   &
              + w4 * ( func ( x - x4 * h,ybar,jpr,mu,sigmam,sigmae,rhom,N ) + func ( x + x4 * h,ybar,jpr,mu,sigmam,sigmae,rhom,N ) ) ) )

errt=1.0e-6

  if ( a == b ) then
    errt = 0.0D+00
    result = 0.0D+00
    return
  end if
 
  if ( icall /= 0 ) then
    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'GAUS8 - Fatal error!'
    write ( *, '(a)' ) '  GAUS8 was called recursively.'
    stop
  end if

  icall = 1
!
!  DIGITS ( X ) = number of base 2 digits in representation of X.
!
  k = digits ( result )

  anib = log10 ( 2.0D+00 ) * real ( k, dp ) / 0.30102000D+00
  nbits = int ( anib )
  nlmx = min ( 30, ( nbits * 5 ) / 8 )
  result = 0.0D+00
  ierr = 1
  ce = 0.0D+00
  result = 0.0D+00
  lmx = nlmx
  lmn = nlmn
 
  if ( b /= 0.0D+00 ) then

    if ( sign ( 1.0D+00, b ) * a <= 0.0D+00 ) then
      go to 10
    end if

    c = abs ( 1.0D+00 - a / b )
    if ( 0.1D+00 < c ) then
      go to 10
    end if

    if ( c <= 0.0D+00 ) then
      go to 140
    end if

    anib = 0.5D+00 - log ( c ) / 0.69314718D+00
    nib = int ( anib )
    lmx = min ( nlmx, nbits-nib-7 )

    if ( lmx < 1 ) then
      go to 130
    end if

    lmn = min ( lmn, lmx )

  end if
 
10    continue
 
  tol = max ( abs ( errt ), 2.0D+00**(5-nbits)) / 2.0D+00
  if ( errt == 0.0D+00 ) then
    tol = sqrt ( epsilon ( 1.0D+00 ) )
  end if

  eps = tol
  hh(1) = ( b - a ) / 4.0D+00
  aa(1) = a
  lr(1) = 1
  l = 1
  est = g8 ( aa(l) + 2.0D+00 * hh(l), 2.0D+00 * hh(l) )
  k = 8
  area = abs ( est )
  ef = 0.5D+00
  mxl = 0
!
!  Compute refined estimates, estimate the error, etc.
!
20 continue
 
  gl = g8 ( aa(l) + hh(l), hh(l) )
  gr(l) = g8 ( aa(l) + 3.0D+00 * hh(l), hh(l) )
  k = k + 16
  area = area + ( abs ( gl ) + abs ( gr(l) ) - abs ( est ) )
 
  glr = gl + gr(l)
  ee = abs ( est - glr ) * ef
  ae = max ( eps * area, tol * abs ( glr ) )

  if ( ee - ae <= 0.0D+00 ) then
    go to 40
  else
    go to 50
  end if
 
30 continue
 
  mxl = 1
 
40 continue
 
  ce = ce + ( est - glr )
 
  if ( lr(l) <= 0 ) then
    go to 60
  else
    go to 80
  end if
!
!  Consider the left half of this level
!
50 continue

  if ( kmx < k ) then
    lmx = kml
  end if

  if ( lmx <= l ) then
    go to 30
  end if

  l = l + 1
  eps = eps * 0.5D+00
  ef = ef / sqrt ( 2.0D+00 )
  hh(l) = hh(l-1) * 0.5D+00
  lr(l) = -1
  aa(l) = aa(l-1)
  est = gl
  go to 20
!
!  Proceed to right half at this level
!
60 continue

  vl(l) = glr

70 continue

  est = gr(l-1)
  lr(l) = 1
  aa(l) = aa(l) + 4.0D+00 * hh(l)
  go to 20
!
!  Return one level
!
80 continue

  vr = glr

90 continue

  if ( l <= 1 ) then
    go to 120
  end if

  l = l - 1
  eps = eps * 2.0D+00
  ef = ef * sqrt ( 2.0D+00 )
 
  if ( lr(l) <= 0 ) then
    vl(l) = vl(l+1) + vr
    go to 70
  else
    vr = vl(l+1) + vr
    go to 90
  end if
!
!  Exit
!
120   continue
 
  result = vr

  if ( mxl == 0 .or. abs ( ce ) <= 2.0D+00 * tol * area ) then
    go to 140
  end if

  ierr = 2
  !write ( *, '(a)' ) ' '
  !write ( *, '(a)' ) 'GAUS8 - Warning!'
  !write ( *, '(a)' ) '  RESULT is probably insufficiently accurate.'
  icall = 0

  if ( errt < 0.0D+00 ) then
    errt = ce
  end if

  return
 
130   continue
 
  ierr = -1
  write ( *, '(a)' ) ' '
  write ( *, '(a)' ) 'GAUS8 - Warning!'
  write ( *, '(a)' ) '  A and B are too close to carry out integration.'
 
140   continue
 
  icall = 0
 
  if ( errt < 0.0D+00 ) then
    errt = ce
  end if
 
  return
end subroutine

!================================================================================!

!same as matlab, creates an array of size n, equally distributed between l and k
function linspace(l,k,n)
USE nrtype

implicit none
REAL(dp), INTENT(IN) :: l, k
REAL(dp) :: d
INTEGER, INTENT(IN) :: n
INTEGER :: i
REAL(dp) :: linspace(n)

d = (k-l)/(n-1)
linspace(1) = Dble(l)
linspace(n) =DBLE(k)
do i = 2,n-1
linspace(i) = linspace(i-1) + d
end do

end function

!================================================================================!

!calculates normal pdf
function normpdf(x)
USE nrtype
implicit none
REAL(dp),INTENT(IN) :: x
REAL(dp) :: normpdf
normpdf=EXP(-(x**2)/2)/SQRT(2.0_dp*pi_d)
end function normpdf

!================================================================================!

!for adda_cooper method
function fadda(x,ybar,jpr,mu,sigmam,sigmae,rhom,N)
USE nrtype
implicit none
INTEGER, INTENT(IN) :: jpr,N
REAL(dp), INTENT(IN) :: x,mu,sigmam,sigmae,rhom,ybar(N+1)
REAL(dp) :: fadda,temp1,temp2

call cumnor((ybar(jpr+1)-mu*(1-rhom)-rhom*x)/sigmae,temp1)
call cumnor((ybar(jpr)-mu*(1-rhom)-rhom*x)/sigmae,temp2)
fadda=normpdf((x-mu)/sigmam)*(temp1-temp2)/sigmam

end function

!================================================================================!

!discretize Ar(1) process using Adda_cooper method
subroutine adda_cooper(N,mu,sigmae,rhom,z,P)

USE nrtype
USE parameters, ONLY: tolbrent

implicit none
REAL(dp), INTENT(IN) :: mu,sigmae,rhom
INTEGER, INTENT(IN) :: N
REAL(dp), INTENT(OUT) :: z(N),P(N,N)
REAL(dp) :: sigmam,ybar(N+1),ybar1(N-1),temp
INTEGER :: j,k
!REAL(dp) :: fadda

sigmam=sigmae/SQRT(1-rhom**2)
!ybar1=linspace((1.0_dp/N),(N-1.0_dp)/N,N-1)
!ybar(1)=-1.0e6
ybar=linspace(0.0_dp,1.0_dp,N+1)
do j=1,N+1
ybar(j)=sigmam*normcdfinv(ybar(j))+mu
end do
ybar(1)=-1.0e+1
ybar(N+1)=1.0e+1
!ybar(N+1)=1.0e+6

do j=1,N
     z(j)=N*sigmam*(normpdf((ybar(j)-mu)/sigmam)-normpdf((ybar(j+1)-mu)/sigmam))+mu
end do

 !Compute the transition matrix P
do j=1,N
    do k=1,N
        call gaus8(fadda,ybar(j),ybar(j+1),temp,ybar,k,mu,sigmam,sigmae,rhom,N)
        P(j,k)=N*temp
    end do
end do

end subroutine adda_cooper

!================================================================================!

!discretize AR(1) process using Tauhen(86) emthod
subroutine tauchen(N,mu,sigmae,rho,z,P)

USE nrtype

implicit none
REAL(dp), INTENT(IN) :: mu,sigmae,rho
INTEGER, INTENT(IN) :: N
REAL(dp), INTENT(OUT) :: z(N), P(N,N)
REAL(dp) :: sigma,temp1,temp2,ybar(N),w
INTEGER :: j,k,m


sigma=sigmae/SQRT(1-rho**2)

m=3
ybar=linspace(mu-m*sigma,mu+m*sigma,N)

w=ybar(2)-ybar(1)

do j=1,N

    call cumnor((ybar(1)-rho*ybar(j)-mu*(1-rho)+w/2)/sigmae,P(j,1))
    do k=2,N-1
        call cumnor((ybar(k)-rho*ybar(j)-mu*(1-rho)+w/2)/sigmae,temp1)
        call cumnor((ybar(k)-rho*ybar(j)-mu*(1-rho)-w/2)/sigmae,temp2)
        P(j,k)=temp1-temp2
    end do
    call cumnor((ybar(N)-rho*ybar(j)-mu*(1-rho)-w/2)/sigmae,temp1)
    P(j,N)=1-temp1
end do

z=ybar

end subroutine


!================================================================================!

!utility function for households
function util_s(c,x)
USE nrtype
USE parameters

implicit none
REAL(dp) :: util_s,c,x
if (sigma==1.0) then
!util_s=LOG(c)+ABS(LOG(c))*x
util_s=LOG(c*(1+x))
else
!util_s=(c**(1-sigma))/(1-sigma)+abs(c**(1-sigma)/(1-sigma))*x
util_s=((c*(1+x))**(1-sigma))/(1-sigma)
end if

end function

!-------------------------------------------------------------------------------!

function util_v(c,x)
USE nrtype
USE parameters

implicit none
REAL(dp) :: c(:),x
REAL(dp) :: util_v(SIZE(c))

if (sigma==1.0) then
!util_v=LOG(c)+ABS(LOG(c))*x
util_v=LOG(c*(1+x))
else
!util_v=(c**(1-sigma))/(1-sigma)+abs(c**(1-sigma)/(1-sigma))*x
util_v=((c*(1+x))**(1-sigma))/(1-sigma)
end if

end function

!-------------------------------------------------------------------------------!

!shift the variables
SUBROUTINE shft(a,b,c,d)
REAL(DP), INTENT(OUT) :: a
REAL(DP), INTENT(INOUT) :: b,c
REAL(DP), INTENT(IN) :: d
a=b
b=c
c=d
END SUBROUTINE shft

!-------------------------------------------------------------------------------!
!locate the index of a variable in an array
FUNCTION locate(xx,x)
USE nrtype
IMPLICIT NONE
REAL(DP), DIMENSION(:), INTENT(IN) :: xx
REAL(DP), INTENT(IN) :: x
INTEGER(I4B) :: locate
INTEGER(I4B) :: n,jl,jm,ju
LOGICAL :: ascnd
	n=size(xx)
	ascnd = (xx(n) >= xx(1))
	jl=1
	ju=n
	do
		if (ju-jl <= 1) exit
		jm=(ju+jl)/2
		if (ascnd .eqv. (x >= xx(jm))) then
			jl=jm
		else
			ju=jm
		end if
	end do
	if (x == xx(1) ) then
		locate=1
	else if (x == xx(n)) then
		locate=n-1
	else
		locate=jl
	end if
END FUNCTION locate

end module
