!  this is revision of mp_trans3. To increase number of simulations, restructures the simulations
!  so that arrays occupy less memory. 
!  Also, divides the transtion into 3 parts: (i) for generations
!  who are alive at the time of transition, (ii) generations who are born after transition,
!  but will be dead by the final steady state (iii) generations who are born after transtion and
!  will be alive by final transtion. 
!  Also, mainsolve1 now stores policy functions to use in solving the simulation of (i) in the  transition

!  mp_transition.f90
!
!  Free-Format Fortran Source File 
!  Generated by PGI Visual Fortran(R)
!  4/21/2011 10:39:06 AM
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! this program solves mortgage pricing (symmetric information) for N-period economy                                                        !
! Utility is assumed as CRRA: c**(1-sigma)/(1-sigma)                                                               !
! Defaulter returns to the market with some prob in the next period.                                               !                                                                                     !
! Includes a subroutine for tauchen and adda-cooper algorithm                                                                      !
! Includes a subroutine for calculating normal cdf                                                                 !
! Value functions are approximated via linear interpolation
! value function for homeowner which is the max of three value functions, is approximated to avoid the severe kinks!
! the simulation part is done through individual simulations. Much useful more micro analysis
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! MODEL
! -Life-cycle model.Certain time horizon.Idiosyncratic shock to income.One-period asset to insure
! -Four tenure states for a household
! -Inactive renter: Renter with default flag. Cannot purchase a house, should rent. 
!                   Decides how much to save and consume
! -Active renter: Can either rent or purchase
!       -Renter: usual consumption/saving problem
!       -Purchaser: should pay the purchase price, but can get a mortgage against the house 
!                   value. Should decide the downpayment. Becomes owner with debt or outright owner next period
! -Outright owner: Owns the house, but has no remaining debt. Can stay or sell the house
!       -Stayer:Ususal consummtpion/saving problem. An outright owner next period.
!       -Seller:Sells the house with some transaction cost. Pays the debt back. 
!               Becomes active renter next period
! -Owner with mortgage:Can either stay, sell or default
!       -Stayer:Makes mortgage payment.Should at least make the minimum periodic mortgage payment.
!               But can pay more.Should finish the debt before starting to save.Becomes an owner with debt 
!               or outright owner depending on how much she pays.
!       -Defaulter:Goes away from the contract with no obligation.Becomes inactive renter.
!                   Usual consume/save problem in current period
! -Mortgages are for life-time.FRM mortgages.Agent can prepay.
! -Mortgages are individual-specific.Terms are determined depending on individual's observables
! -Need to keep track of continuation value of contract. 


!============================================================================================================!
program main

USE nrtype
use parameters !, only:mk,rmmaxa,age,pwrr,Nr,Nnu,Na,age,Nd,Nr,ret,tfinal,Nsim
USE global_parameters
use global
use procedures, only:linspace,locate
use subroutines, only:myzbrent2
!use omp_lib
!use bcpol_int

implicit none

REAL(dp) :: ftol1,ax,cx,own_rate,rmgrid_temp(Nr),tbegin,tend
real(dp), allocatable, dimension(:) :: phtguess,own
real(dp), EXTERNAL :: fcn
external fcn6
integer i,t0,tmed0,tmed,t,tf

call cpu_time(tbegin)

!tbegin=omp_get_wtime()

!adjust the time period
!write(*,*) 'enter the period length in terms of years (should be a divider of 18):'
!read(*,*) cons
!if (mod(18,cons)/=0) then
!print*,'should be a divider of 18'
!pause
!end if
age=45
ret=25
tmed0=2
tmed=5
tfinal=age+tmed+1
own_star=0.66_dp

allocate(agrid(Na,age+1),amax(age+1),ydet(age))
allocate(agrid2(Na2,age+1))
allocate(pht(-age:tfinal+age),rmm(-age:tfinal+age))
allocate(ram(3),phs(2))

ram(1)=0.02_dp
ram(2)=0.03_dp
ram(3)=0.005d0
unempM(1)=0.05_dp
unempM(2)=0.06_dp
mindown(1)=0.0_dp
mindown(2)=0.2_dp
!ram(3)=0.01_dp
!initialize the variables
r=minval(ram)
rmk=r+mk
rmmax=maxval(ram)+0.04+mk
rmgrid(1:Nr/4+1)=(linspace(0.0_dp,(ram(1)+mk-rmk)**(1/pwrr),Nr/4+1))**pwrr+rmk
rmgrid(Nr/4+1:Nr/2+1)=(linspace(0.0_dp,(ram(2)-ram(1)-mk)**(1/pwrr),(Nr/2-Nr/4+1)))**pwrr+ram(1)+mk
rmgrid(Nr/2+1:Nr)=(linspace(0.0_dp,(rmmax-ram(2)-mk)**(1/pwrr),Nr-Nr/2+1))**pwrr+ram(2)
!print*,'initialize the variables'
!unemp=unempM(1)
!call initial

!ax=1_dp
!cx=6_dp
!ftol1 = 1.0E-5

!open(2,file='phs.txt')
!do i=1,2
!read(2,*) phs(i)
!end do
!close(2)
!
do i=1,size(ram)
r=ram(i)
rmk=r+mk
rmk_loc=locate(rmgrid,rmk)
pgrid=(linspace(0.0_dp,(1.0_dp-mindown(i))**(1/pwrp),Np))**pwrp+mindown(i) !downpayment
unemp=unempM(i)
call initial
print*,'interest rate=',r
!ph=phs(i)
!call mainsolve(own_rate)
phs(i)=myzbrent2(fcn,ax,cx,ftol1)
print*,'steady-state house price=',phs(i)
end do
!
!pause

open(2,file='phs.txt',status='replace')
do i=1,size(phs)
write(2,*) phs(i)
end do
close(2)

!pause

!open(2,file='phs.txt')
!do i=1,2
!read(2,*) phs(i)
!end do
!close(2)

!ph=phs(1)
!dgrid=-ph*(1-dgrid1)
!call mainsolve(own_rate)
!pause

pht(-age:-1)=phs(1)
rmm(-age:-1)=ram(1)

allocate(ownA(-age:tfinal+age),rentA(-age:tfinal+age),defA(-age:tfinal+age),ownM(-age:tfinal+age))
allocate(foreA(-age:tfinal+age),buyerA(-age:tfinal+age),premA(-age:tfinal+age),downA(-age:tfinal+age))
allocate(negeqA(-age:tfinal+age),debtA(-age:tfinal+age),profA(-age:tfinal+age),loanA(-age:tfinal+age))

rentA=0
defA=0
foreA=0
buyerA=0
premA=0
downA=0
ownA=0
ownM=0
negeqA=0.0_dp
debtA=0.0d0
profA=0.0d0
loanA=0.0d0

allocate(ah_init(Nr,0:Nnu,Nd,age),sth_init(Nr,0:Nnu,Nd,age))
allocate(am_init(0:Nnu,Na+Nd,age),stm_init(0:Nnu,Na+Nd,age))
allocate(ar_init(0:Nnu,Na,age),ad_init(0:Nnu,Na,age),ahz_init(0:Nnu,Na,age))
allocate(str_init(0:Nnu,Na,age),sthz_init(0:Nnu,Na,age))
allocate(phi_init(0:Nnu,Na,age),rm_init(0:Nnu,Na,age))

allocate(ah_init2(Nr,0:Nnu,Nd,age,-age:tmed0-1),sth_init2(Nr,0:Nnu,Nd,age,-age:tmed0-1))
allocate(am_init2(0:Nnu,Na+Nd,age,-age:tmed0-1),stm_init2(0:Nnu,Na+Nd,age,-age:tmed0-1))
allocate(ar_init2(0:Nnu,Na,age,-age:tmed0-1),ad_init2(0:Nnu,Na,age,-age:tmed),ahz_init2(0:Nnu,Na,age,-age:tmed0-1))
allocate(str_init2(0:Nnu,Na,age,-age:tmed0-1),sthz_init2(0:Nnu,Na,age,-age:tmed0-1))
allocate(phi_init2(0:Nnu,Na,age,-age:tmed0-1),rm_init2(0:Nnu,Na,age,-age:tmed0-1))

r=ram(1)
rmk=r+mk
rmk_loc=locate(rmgrid,rmk)
ph=phs(1)
pgrid=(linspace(0.0_dp,(1.0_dp-mindown(1))**(1/pwrp),Np))**pwrp+mindown(1) !downpayment
unemp=unempM(1)
call initial
call mainsolve1(-age)

do t=-age+1,-1
ah_init2(:,:,:,:,t)=ah_init
am_init2(:,:,:,t)=am_init
ar_init2(:,:,:,t)=ar_init
ad_init2(:,:,:,t)=ad_init
ahz_init2(:,:,:,t)=ahz_init
str_init2(:,:,:,t)=str_init
sth_init2(:,:,:,:,t)=sth_init
stm_init2(:,:,:,t)=stm_init
sthz_init2(:,:,:,t)=sthz_init
phi_init2(:,:,:,t)=phi_init(:,:,:)
rm_init2(:,:,:,t)=rm_init(:,:,:)
end do

t0=0
tf=tfinal-1

r=ram(2)
rmk=r+mk
rmk_loc=locate(rmgrid,rmk)
ph=phs(2)
pgrid=(linspace(0.0_dp,(1.0_dp-mindown(2))**(1/pwrp),Np))**pwrp+mindown(2) !downpayment
unemp=unempM(2)
call initial
call mainsolve6(tfinal+age)

!allocate value functions for final steady state
!need these so that I don't have to solve the remaining lifetime problem of the transiting agent after the setady state
allocate(vrlast(0:Nnu,Na,age),vdlast(0:Nnu,Na,age),vhzlast(0:Nnu,Na,age))
allocate(vmlast(0:Nnu,Na+Nd,age),vlmlast(0:Nnu,Nd,age))
allocate(vhlast(Nr,0:Nnu,Nd,age),vllast(Nr,0:Nnu,Nd,age))

allocate(ah_last(Nr,0:Nnu,Nd,age),sth_last(Nr,0:Nnu,Nd,age))
allocate(am_last(0:Nnu,Na+Nd,age),stm_last(0:Nnu,Na+Nd,age))
allocate(ar_last(0:Nnu,Na,age),ad_last(0:Nnu,Na,age),ahz_last(0:Nnu,Na,age))
allocate(str_last(0:Nnu,Na,age),sthz_last(0:Nnu,Na,age))
allocate(phi_last(0:Nnu,Na,age),rm_last(0:Nnu,Na,age))
!solve for final steady-state value functions
!print*,'solve for final steady-state value functions'
call mainsolve4
!solve for transition prices

open(2,file='pht2.txt',status='replace')
close(2)
open(2,file='own2.txt',status='replace')
close(2)
open(2,file='iter2.txt',status='replace')
write(2,*) 0
close(2)

pht(tf+1:tfinal+age)=phs(2)
rmm(t0:tfinal+age)=ram(2)

open(2,file='pmin_init.txt')
do i=t0,tf
read(2,*) pht(i)
end do
close(2)

print*,'solve policy functions after the initial shock before the FED policy'

do t=t0-age,tmed0-1
call mainsolve7(t,t0)
end do

allocate(phtguess(tmed0:tf),own(tmed0+1:tf+1))

!open(2,file='pmin2.txt')
!do i=tmed0,tf
!read(2,*) phtguess(i)
!end do
!close(2)
!phtguess=linspace(phtguess(1),phtguess(tfinal-1),tfinal-1)
phtguess=pht

rmm(tmed0:tmed-1)=ram(3)

open(2,file='ramt2.txt',status='replace')
do i=-age,tfinal+age
write(2,*) rmm(i)
end do
close(2)

!print*,'solving transition'
!call fcn6(tmed0,tf,phtguess,own)

print*,'solve for transition prices'
call my_est(fcn6,tmed0,tfinal-1,phtguess)

open(2,file='ownc2.txt',status='replace')
do i=1,tfinal
write(2,*) own(i)
end do
close(2)

deallocate(phtguess,own)

r=ram(1)
rmk=r+mk
rmk_loc=locate(rmgrid,rmk)
ph=phs(1)
pgrid=(linspace(0.0_dp,(1.0_dp-mindown(1))**(1/pwrp),Np))**pwrp+mindown(1) !downpayment
unemp=unempM(1)
call initial
call mainsolve6(-1)

open(2,file='ownA.txt',status='replace')
do i=-1,tfinal
write(2,*) ownA(i)
end do
write(2,*) ownA(tfinal+age)
close(2)
open(2,file='ownM.txt',status='replace')
do i=-1,tfinal
write(2,*) ownM(i)
end do
write(2,*) ownM(tfinal+age)
close(2)
open(2,file='rentA.txt',status='replace')
do i=-1,tfinal
write(2,*) rentA(i)
end do
write(2,*) rentA(tfinal+age)
close(2)
open(2,file='defA.txt',status='replace')
do i=-1,tfinal
write(2,*) defA(i)
end do
write(2,*) defA(tfinal+age)
close(2)
open(2,file='foreA.txt',status='replace')
do i=-1,tfinal
write(2,*) foreA(i)
end do
write(2,*) foreA(tfinal+age)
close(2)
open(2,file='buyerA.txt',status='replace')
do i=-1,tfinal
write(2,*) buyerA(i)
end do
write(2,*) buyerA(tfinal+age)
close(2)
open(2,file='premA.txt',status='replace')
do i=-1,tfinal
write(2,*) premA(i)
end do
write(2,*) premA(tfinal+age)
close(2)
open(2,file='downA.txt',status='replace')
do i=-1,tfinal
write(2,*) downA(i)
end do
write(2,*) downA(tfinal+age)
close(2)
open(2,file='negeqA.txt',status='replace')
do i=-1,tfinal
write(2,*) negeqA(i)
end do
write(2,*) negeqA(tfinal+age)
close(2)
open(2,file='debtA.txt',status='replace')
do i=-1,tfinal
write(2,*) debtA(i)
end do
write(2,*) debtA(tfinal+age)
close(2)
open(2,file='profA.txt',status='replace')
do i=-1,tfinal
write(2,*) profA(i)
end do
write(2,*) profA(tfinal+age)
close(2)
open(2,file='loanA.txt',status='replace')
do i=-1,tfinal
write(2,*) loanA(i)
end do
write(2,*) loanA(tfinal+age)
close(2)


deallocate(ownA,rentA,defA,ownM)
deallocate(foreA,buyerA,premA,downA,debtA,profA,loanA,negeqA)


deallocate(pht,ram,phs,rmm)
deallocate(agrid,amax,ydet,agrid2)

deallocate(vrlast,vdlast,vhzlast,vmlast,vlmlast,vhlast,vllast)
deallocate(ah_last,am_last,ar_last,ad_last,ahz_last)
deallocate(str_last,sth_last,stm_last,sthz_last,phi_last,rm_last)
deallocate(ah_init,am_init,ar_init,ad_init,ahz_init)
deallocate(str_init,sth_init,stm_init,sthz_init,phi_init,rm_init)

call cpu_time(tend)
!tend=omp_get_wtime()
print*,'time elapsed=',tbegin-tend

end program


